from univariate.anomaly.detector.anomaly_detector import AnomalyDetector
from univariate.anomaly.detector.iqr_outlier_detector import IQROutlierDetector
from univariate.anomaly.detector.residual_anomaly_detector import ResidualAnomalyDetector
