"""
    DTO for reporting analysis result
"""


class AnalysisReport:
    """
    DTO for reporting analysis result
    """

    def __init__(self):
        """
        # todo: temp report, spec latter
        """
        self.metrics = dict()
        self.parameters = dict()
        self.plots = dict()
